
/*
example-2.js

Description:
    Start web server on port 3000
    Open HTML browser to:
        http://localhost:3000/
    Result:
        A rectangle form
            editable: w, h
            read-only: a, p (clear on any edit)
            action: solve

Components:

    WebServer - this file - start web server
    Router - this file - route incoming requests to middleware
    Middleware - this file - handle requests
*/

const
    http = require('http'),
    htmlType = {'Content-Type': 'text/html'},
    express = require('express'),
    app = express(),
    log = console.log;

class Rec {
    w(arg) { if (arg !== undefined) this._w = arg; return this._w; }
    h(arg) { if (arg !== undefined) this._h = arg; return this._h; }
    constructor(w, h) {
        this.w(w ?? 0);
        this.h(h ?? 0);
    }
    a() { return this.w() * this.h(); }
    p() { return 2 * (this.w() + this.h()); }
    factor(f) { this._w *= f; this._h *= f; }
}

class AppStartupTools {

	static initializeApp(app) {
		app.use(express.urlencoded({extended: false}));
		app.use(express.json());
		app.use(this.logRequest);
		app.set("port", process.env.PORT || 3000);
		app.set("view engine", "ejs");
		//sets "public" as root of static resources
		app.use(express.static("public"));
	}

	static logRequest(req, res, next) {
		//middleware function (helpful for debugging)
		log(`Request received: ${req.url}`);
		log(`Request method: ${req.method}`);
		log(`Request Body: ${JSON.stringify(req.body)}`);
		next();
	}

}

class Middleware {
    static home(req, res) {
        res.render('index');
    }
    static examples(req, res) {
        res.render('examples');
    }
    static playground(req, res) {
        res.render('playground');
    }
}


class Router {
    static setupRoutes(app) {
        app.get('/', Middleware.home);
        app.get('/examples', Middleware.examples);
        app.get('/playground', Middleware.playground);
    }
}

class WebServer {

    setup() {
        AppStartupTools.initializeApp(app);
        Router.setupRoutes(app);
    }

    start() {
        this.setup();
        const port = app.get('port');
        app.listen(app.get('port'), () => log(`web server started on port ${port}`));
    }

}

const webServer = new WebServer();
webServer.start();
